/******************************************************************

        MUSCLE SmartCard Development ( http://www.linuxnet.com )
            Title  : kpub.c
            Package: RSA Smartcard PAM module
            Author : David Corcoran
            Date   : 4/12/00
            Purpose: This extracts and encrypts using RSA public keys.

********************************************************************/

#include <includes.h>
#include <des.h>
#include <ssh.h>
#include <gmp.h>
#include <xmalloc.h>
#include <rsa.h>
#include <stdio.h>
#include <winscard.h>

void fatal(const char *fmt, ...)
{
  va_list ap;
  char buf[1024];

  va_start(ap, fmt);
  vsnprintf(buf, sizeof(buf), fmt, ap);
  va_end(ap);
  fprintf(stderr, "%s\n", buf);
  exit(255);
}

void RSAKpubGetKey( RSAPublicKey *key, char *file ) {

  int i, fd;
  unsigned char buf[1024];

  key->bits = 1024;
  mpz_init(&key->e);
  mpz_init(&key->n);

  fd = open(file, O_RDONLY); 

  if ( fd <= 0 ) {
    printf("Public key file %s does not exist\n", file);
    exit(1);
  }

  read( fd, buf, 4 );
  mp_unlinearize_msb_first (&key->e, buf, 4);
  read( fd, buf, 128 );
  mp_unlinearize_msb_first (&key->n, buf, 128);
  close(fd);

}

void RSAKpubEncrypt( RSAPublicKey *key, PUCHAR pucIBuffer, 
		     PUCHAR pucOBuffer, LPDWORD pdwSize ) {

  LONG rv;
  MP_INT input;
  MP_INT output;

  mpz_init( &input );
  mpz_init( &output );

  mp_unlinearize_msb_first (&input, pucIBuffer, *pdwSize);
  rsa_public( &output, &input, key );
  mp_linearize_msb_first( pucOBuffer, *pdwSize, &output );

  mpz_clear( &input );
  mpz_clear( &output );
}
